/*----------------------------------------------------------------------
"Debugging Applications" (Microsoft Press)
Copyright (c) 1997-2000 John Robbins -- All rights reserved.
----------------------------------------------------------------------*/
#include "PCH.h"
#include "DebugInterface.h"
#include "DebuggeeInfo.h"
#include "ModuleNotification.h"
#include "CriticalSection.h"

/*//////////////////////////////////////////////////////////////////////
                     File Scope Class Declarations
//////////////////////////////////////////////////////////////////////*/
class CModNotification
{
public  :
    CDebugBaseUser * m_pUserClass ;
    LPDEBUGGEEINFO   m_pData      ;
    DWORD            m_dwBaseAddr ;

    CModNotification ( CDebugBaseUser * pUserClass ,
                       LPDEBUGGEEINFO   pData      ,
                       DWORD            dwBaseAddr  )
    {
        m_pUserClass = pUserClass ;
        m_pData      = pData      ;
        m_dwBaseAddr = dwBaseAddr ;
    }
} ;

typedef vector<CModNotification> MODNOTVECT ;

/*//////////////////////////////////////////////////////////////////////
                      File Scope Global Variables
//////////////////////////////////////////////////////////////////////*/
// The module critical section.  At this point, the debugger thread is
// the only thread accessing this, but I could change this in the future
// so I wanted to be prepared.
static CCriticalSection g_CS ;
// The module notification list.
static MODNOTVECT g_vMods ;

/*//////////////////////////////////////////////////////////////////////
                     !!Implementation Starts Here!!
//////////////////////////////////////////////////////////////////////*/
void AddModuleNotification ( CDebugBaseUser * pUserClass ,
                             LPDEBUGGEEINFO   pData      ,
                             DWORD            dwBaseAddr  )
{
    // Always grab the critical section.
    CUseCriticalSection cUse ( g_CS ) ;

    CModNotification cMod ( pUserClass , pData , dwBaseAddr ) ;

    g_vMods.push_back ( cMod ) ;
}

// The reason for this function is that I have seen cases where a DLL
// gets loaded and unloaded very rapidly.  I see it with WinWord and my
// printer driver.  Since the functions in this module are after the
// fact of the load, this function keeps the data in synch with the
// actual modules.
void RemoveModuleNotification ( CDebugBaseUser * pUserClass ,
                                LPDEBUGGEEINFO   pData      ,
                                DWORD            dwBaseAddr  )
{
    // Always grab the critical section.
    CUseCriticalSection cUse ( g_CS ) ;

    MODNOTVECT::iterator i ;
    for ( i = g_vMods.begin ( ) ;
          i != g_vMods.end ( )  ;
          i++                    )
    {
        if ( ( dwBaseAddr == i->m_dwBaseAddr ) &&
             ( pData      == i->m_pData      ) &&
             ( pUserClass == i->m_pUserClass )    )
        {
            g_vMods.erase ( i ) ;
            break ;
        }
    }
}


DWORD PulseModuleNotification ( void )
{
    // Always grab the critical section.
    CUseCriticalSection cUse ( g_CS ) ;

    DWORD dwRet = DBG_CONTINUE ;

    // Do the work....
    while ( g_vMods.size ( ) > 1 )
    {
        // Start from the front of the vector.
        MODNOTVECT::iterator i = g_vMods.begin ( ) ;

        IMAGEHLP_MODULE stModInfo ;
        memset ( &stModInfo , NULL , sizeof ( IMAGEHLP_MODULE ) ) ;
        stModInfo.SizeOfStruct = sizeof ( IMAGEHLP_MODULE ) ;

        TCHAR szName [ MAX_PATH ] ;

        BOOL bSymGetMod =
          i->m_pData->SymGetModuleInfo ( (DWORD)i->m_dwBaseAddr      ,
                                         &stModInfo                  ,
                                         szName                      ,
                                         sizeof(szName)/sizeof(TCHAR) );
        if ( TRUE == bSymGetMod )
        {
            // Cool, it worked, report the info to the user.
            if ( 0 ==
                    i->m_pUserClass->
                        LoadedDllInfo ( i->m_pData->GetProcessId ( ) ,
                                        szName                       ,
                                        i->m_dwBaseAddr              ,
                                        &stModInfo                    ))
            {
                // A serious synchronization problem occurred!
                dwRet = 0 ;
                break ;
            }
            // Get rid of this item in the array.
            g_vMods.erase ( i ) ;
        }
        else
        {
            // Try again later.
            break ;
        }
    }
    return ( dwRet ) ;
}

void ClearModuleNotification ( void )
{
    g_vMods.clear ( ) ;
}
