/*----------------------------------------------------------------------
"Debugging Applications" (Microsoft Press)
Copyright (c) 1997-2000 John Robbins -- All rights reserved.
----------------------------------------------------------------------*/
/*//////////////////////////////////////////////////////////////////////

Unfortunately, there is no documented way to turn an HANDLE into its
complete filename when the module is not completely mapped into memory.
PSAPI and TOOLHELP32 functions fail on Windows 2000 in these situations.
While there are fields in the debugger API structures for the filenames,
they are never filled out.  Obviously, the OS loader has this
information, but MS does not want to give it to us.

This information might be in the PDB symbol files, but that is
undocumented as well.  Consequently, I hack around all of this by
keeping the module information around and checking if I can read the
file in memory with the PSAPI or TOOLHELP32 information.  The symbols
are already loaded so the information is there to set breakpoints.  I
just wanted to duplicate the VC Debugger and show the full modules
loaded.

//////////////////////////////////////////////////////////////////////*/
#ifndef _MODULENOTIFICATION_H
#define _MODULENOTIFICATION_H


/*----------------------------------------------------------------------
FUNCTION    :   AddModuleNotification.
DISCUSSION  :
    Adds a module that needs a notification.
PARAMETERS  :
    pUserClass - The user's synchronization and debug class.
    pData      - The debuggee information.
    dwBaseAddr - The module base address that needs the notification.
RETURNS     :
    Nothing.
----------------------------------------------------------------------*/
void AddModuleNotification ( CDebugBaseUser * pUserClass ,
                             LPDEBUGGEEINFO   pData      ,
                             DWORD            dwBaseAddr  ) ;

/*----------------------------------------------------------------------
FUNCTION    :   RemoveModuleNotification.
DISCUSSION  :
    Removes a module that needs notification.
PARAMETERS  :
    pUserClass - The user's synchronization and debug class.
    pData      - The debuggee information.
    dwBaseAddr - The module base address that needs to be removed.
RETURNS     :
    Nothing.
----------------------------------------------------------------------*/
void RemoveModuleNotification ( CDebugBaseUser * pUserClass ,
                                LPDEBUGGEEINFO   pData      ,
                                DWORD            dwBaseAddr  ) ;

/*----------------------------------------------------------------------
FUNCTION    :   PulseModuleNotification
DISCUSSION  :
    Does the work of checking for the full path names of modules loaded
with AddModuleNotification.
PARAMETERS  :
    None.
RETURNS     :
    0 - There was a serious problem and the debug thread should end.
----------------------------------------------------------------------*/
DWORD PulseModuleNotification ( void ) ;

/*----------------------------------------------------------------------
FUNCTION    :   ClearModuleNotification
DISCUSSION  :
    Clears out any remaining items in the notification list.  This
should be called prior to the debug loop shutting down.
PARAMETERS  :
    None.
RETURNS     :
    Nothing.
----------------------------------------------------------------------*/
void ClearModuleNotification ( void ) ;



#endif  // _MODULENOTIFICATION_H

