/*----------------------------------------------------------------------
"Debugging Applications" (Microsoft Press)
Copyright (c) 1997-2000 John Robbins -- All rights reserved.
----------------------------------------------------------------------*/
#include "PCH.h"
#include "DebugInterface.h"
#include "DebuggeeInfo.h"
#include "ProcessDebugEvents.h"
#include "ModuleNotification.h"

/*//////////////////////////////////////////////////////////////////////
                        File Scope Declarations
//////////////////////////////////////////////////////////////////////*/
// Handler for CREATE_PROCESS_DEBUG_EVENTs
static
DWORD CreateProcessEvent ( CDebugBaseUser *            pUserClass  ,
                           LPDEBUGGEEINFO              pData       ,
                           DWORD                       dwProcessId ,
                           DWORD                       dwThreadId  ,
                           CREATE_PROCESS_DEBUG_INFO & stCPDI       ) ;
// Handler for OUTPUT_DEBUG_STRING_EVENTs
static
DWORD OutputDebugStringEvent ( CDebugBaseUser *           pUserClass  ,
                               LPDEBUGGEEINFO             pData       ,
                               DWORD                      dwProcessId ,
                               DWORD                      dwThreadId  ,
                               OUTPUT_DEBUG_STRING_INFO & stODSI     ) ;
// Handler for RIP_EVENTs
static
DWORD RipEvent ( CDebugBaseUser * pUserClass  ,
                 LPDEBUGGEEINFO   pData       ,
                 DWORD            dwProcessId ,
                 DWORD            dwThreadId  ,
                 RIP_INFO &       stRI         ) ;
static
DWORD ExceptionEvent ( CDebugBaseUser *       pUserClass  ,
                       LPDEBUGGEEINFO         pData       ,
                       DWORD                  dwProcessId ,
                       DWORD                  dwThreadId  ,
                       EXCEPTION_DEBUG_INFO & stEDI        ) ;


/*//////////////////////////////////////////////////////////////////////
                     !!IMPLEMENTATION STARTS HERE!!
//////////////////////////////////////////////////////////////////////*/

DWORD ProcessDebugEvents ( CDebugBaseUser * pUserClass ,
                           LPDEBUGGEEINFO   pData      ,
                           LPDEBUG_EVENT    pDE         )
{
    // Call me paranoid....
    ASSERT ( FALSE == IsBadReadPtr ( pUserClass ,
                                     sizeof ( CDebugBaseUser ) ) ) ;
    ASSERT ( FALSE == IsBadReadPtr ( pDE , sizeof ( LPDEBUG_EVENT ) ) );
    ASSERT ( NULL != pData ) ;

    // The return value from the handler.
    DWORD dwRet DBG_CONTINUE ;

    switch ( pDE->dwDebugEventCode )
    {
        case EXCEPTION_DEBUG_EVENT       :
            dwRet = ExceptionEvent ( pUserClass       ,
                                     pData            ,
                                     pDE->dwProcessId ,
                                     pDE->dwThreadId  ,
                                     pDE->u.Exception  ) ;
            break ;
        case CREATE_THREAD_DEBUG_EVENT   :
            {
                dwRet =
                    pUserClass->CreateThreadEvent( pDE->dwProcessId    ,
                                                   pDE->dwThreadId     ,
                                                   pDE->u.CreateThread);
            }
            break ;
        case CREATE_PROCESS_DEBUG_EVENT  :
            dwRet = CreateProcessEvent ( pUserClass               ,
                                         pData                    ,
                                         pDE->dwProcessId         ,
                                         pDE->dwThreadId          ,
                                         pDE->u.CreateProcessInfo  ) ;
            break ;
        case EXIT_THREAD_DEBUG_EVENT     :
            {
                dwRet =
                    pUserClass->ExitThreadEvent ( pDE->dwProcessId ,
                                                  pDE->dwThreadId  ,
                                          pDE->u.ExitThread.dwExitCode);
            }
            break ;
        case EXIT_PROCESS_DEBUG_EVENT    :
            {
                pUserClass->ExitProcessEvent ( pDE->dwProcessId       ,
                                               pDE->dwThreadId        ,
                                       pDE->u.ExitProcess.dwExitCode  );
            }
            break ;
        case LOAD_DLL_DEBUG_EVENT        :
            {
                pData->SymLoadModule ( pDE->u.LoadDll.hFile            ,
                                     (DWORD)pDE->u.LoadDll.lpBaseOfDll);

                pUserClass->LoadDllEvent ( pDE->dwProcessId ,
                                           pDE->dwThreadId  ,
                                           pDE->u.LoadDll    ) ;

                // Add this module to the notification chain.
                AddModuleNotification ( pUserClass  ,
                                        pData       ,
                                     (DWORD)pDE->u.LoadDll.lpBaseOfDll);
            }
            break ;
        case UNLOAD_DLL_DEBUG_EVENT      :
            {
                pUserClass->UnloadDllEvent ( pDE->dwProcessId          ,
                                             pDE->dwThreadId           ,
                                          pDE->u.UnloadDll.lpBaseOfDll);

                pData->SymUnloadModule (
                                (DWORD)pDE->u.UnloadDll.lpBaseOfDll ) ;
                // Just in case this was one of those load and immediate
                // unload things...
                RemoveModuleNotification ( pUserClass ,
                                           pData      ,
                                   (DWORD)pDE->u.UnloadDll.lpBaseOfDll);
            }
            break ;
        case OUTPUT_DEBUG_STRING_EVENT   :
            dwRet = OutputDebugStringEvent ( pUserClass         ,
                                             pData              ,
                                             pDE->dwProcessId   ,
                                             pDE->dwThreadId    ,
                                             pDE->u.DebugString   ) ;
            break ;
        case RIP_EVENT                   :
            dwRet = RipEvent ( pUserClass       ,
                               pData            ,
                               pDE->dwProcessId ,
                               pDE->dwThreadId  ,
                               pDE->u.RipInfo    ) ;
            break ;
        default                          :
            {
                ASSERT ( !"Unknown debug event!" ) ;

                pUserClass->Output ( "An unknown debugger event "
                                     "occurred ->%i (0x%08X)" ,
                                     pDE->u.RipInfo.dwType    ,
                                     pDE->u.RipInfo.dwType     ) ;
                pUserClass->OutputCRLF ( ) ;
            }
            break ;
    }
    return ( dwRet ) ;
}


static
DWORD CreateProcessEvent ( CDebugBaseUser *            pUserClass  ,
                           LPDEBUGGEEINFO              pData       ,
                           DWORD                       dwProcessId ,
                           DWORD                       dwThreadId  ,
                           CREATE_PROCESS_DEBUG_INFO & stCPDI       )
{

    // Set the process handle, which is the first thing that has to
    // happen.
    pData->SetProcessHandle ( stCPDI.hProcess ) ;
    // The process ID.
    pData->SetProcessId ( dwProcessId ) ;

    // Initialize the symbol engine.
    pData->InitializeSymEng ( ) ;
    // Load the first module.
    pData->SymLoadModule ( stCPDI.hFile ,
                          (DWORD)stCPDI.lpBaseOfImage ) ;

    // Do the informational output.
    IMAGEHLP_MODULE stModInfo ;
    memset ( &stModInfo , NULL , sizeof ( IMAGEHLP_MODULE ) ) ;
    stModInfo.SizeOfStruct = sizeof ( IMAGEHLP_MODULE ) ;
    pData->SymGetModuleInfo ( (DWORD)stCPDI.lpBaseOfImage ,
                              &stModInfo                   ) ;

    DWORD dwRet = DBG_CONTINUE ;

    // Send the initial create process event.
    dwRet = pUserClass->CreateProcessEvent ( dwProcessId ,
                                             dwThreadId  ,
                                             stCPDI       ) ;
    if ( DBG_CONTINUE != dwRet )
    {
        return ( dwRet ) ;
    }
    // Do the informational posting to the user.
    dwRet = pUserClass->LoadedDllInfo ( dwProcessId                 ,
                                        pData->GetProcessName ( )   ,
                                        (DWORD)stCPDI.lpBaseOfImage ,
                                        &stModInfo                   ) ;
    if ( DBG_CONTINUE != dwRet )
    {
        return ( dwRet ) ;
    }
    // Finally, fake a little thread creation so that it is easier to
    // keep the thread information straight.
    CREATE_THREAD_DEBUG_INFO stCTI ;
    stCTI.hThread           = stCPDI.hThread ;
    stCTI.lpStartAddress    = stCPDI.lpStartAddress ;
    stCTI.lpThreadLocalBase = stCPDI.lpThreadLocalBase ;

    dwRet = pUserClass->CreateThreadEvent ( dwProcessId ,
                                            dwThreadId  ,
                                            stCTI        ) ;
    return ( dwRet ) ;
}

static
DWORD OutputDebugStringEvent ( CDebugBaseUser *           pUserClass  ,
                               LPDEBUGGEEINFO             pData       ,
                               DWORD                      dwProcessId ,
                               DWORD                      dwThreadId  ,
                               OUTPUT_DEBUG_STRING_INFO & stODSI     )
{
    TCHAR szBuff[ 512 ] ;

    HANDLE hProc = pData->GetProcessHandle ( ) ;

    DWORD dwRead ;

    // Read the memory.
    BOOL bRet = DBG_ReadProcessMemory( hProc                           ,
                                       stODSI.lpDebugStringData        ,
                                       szBuff ,
                                       min ( sizeof ( szBuff )         ,
                                             stODSI.nDebugStringLength),
                                       &dwRead                        );
    ASSERT ( TRUE == bRet ) ;
    if ( TRUE == bRet )
    {
        // Always NULL terminate the string.
        szBuff [ dwRead + 1 ] = _T ( '\0' ) ;
        // Convert CR/LFs if I'm supposed to.
        pUserClass->ConvertCRLF ( szBuff , sizeof ( szBuff ) ) ;
        // Send the converted string on to the user class.
        pUserClass->OutputDebugStringEvent ( dwProcessId ,
                                             dwThreadId  ,
                                             szBuff       ) ;
    }

    return ( DBG_CONTINUE ) ;
}

static
DWORD RipEvent ( CDebugBaseUser * pUserClass  ,
                 LPDEBUGGEEINFO   /*pData*/       ,
                 DWORD            /*dwProcessId*/ ,
                 DWORD            /*dwThreadId*/  ,
                 RIP_INFO &       stRI         )
{
    // Has anyone ever seen one of these?

    TCHAR szLevel [ 25 ] ;
    switch ( stRI.dwType )
    {
        case SLE_ERROR       :
            _tcscpy ( szLevel , _T ( "SLE_ERROR"  ) ) ;
            break ;
        case SLE_MINORERROR  :
            _tcscpy ( szLevel , _T ( "SLE_MINORERROR" ) ) ;
            break ;
        case SLE_WARNING     :
            _tcscpy ( szLevel , _T ( "SLE_WARNING" ) ) ;
            break ;
        case 0               :
            _tcscpy ( szLevel , _T ( "None" ) ) ;
            break ;
        default              :
            wsprintf ( szLevel , _T ( "Unknown %i" ) , stRI.dwType ) ;
            break ;
    }
    pUserClass->Output ( "RIP Event, level %s" , szLevel ) ;
    // Get the message from the system.  We ask it to allocate the
    // message buffer so we must make sure to delete it.
    LPTSTR szMsg = NULL ;
    BOOL bAllocated = TRUE ;
    if ( 0 == FormatMessage ( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                              FORMAT_MESSAGE_FROM_SYSTEM      ,
                              NULL                            ,
                              stRI.dwError                    ,
                              MAKELONG ( MAKELANGID ( LANG_NEUTRAL ,
                                                  SUBLANG_NEUTRAL ) ,
                                         0                        ) ,
                              szMsg                           ,
                              0                               ,
                              NULL                             ) )
    {
        szMsg = _T ( "FormatMessage failed" ) ;
        bAllocated = FALSE ;
    }
    pUserClass->Output ( _T ( "   %s" ) , szMsg ) ;
    pUserClass->OutputCRLF ( ) ;
    if ( TRUE == bAllocated )
    {
        LocalFree ( szMsg ) ;
    }

    return ( DBG_CONTINUE ) ;
}


static
DWORD ExceptionEvent ( CDebugBaseUser *       pUserClass  ,
                       LPDEBUGGEEINFO         pData       ,
                       DWORD                  dwProcessId ,
                       DWORD                  dwThreadId  ,
                       EXCEPTION_DEBUG_INFO & stEDI        )
{
    if ( ( EXCEPTION_BREAKPOINT ==
                stEDI.ExceptionRecord.ExceptionCode ) &&
         ( FALSE == pData->GetLoaderBreakpoint ( ) )     )
    {
        pData->SetLoaderBreakpoint ( ) ;
    }
    return ( pUserClass->ExceptionEvent ( dwProcessId ,
                                          dwThreadId  ,
                                          stEDI        ) ) ;
}

